#!/usr/bin/local/python2.4
from random import randint  #random integer function, useful for test
from time import sleep
import signal #for interupting mapping sequence
from matplotlib import rcParams
import numarray as na
import pylab as pl
from pylab import cm
import common
import IRCamSocket, IRCamCommands
import mircFiberExplorerXchan
import time
import pdb

class Stage_Info:
    def __init__(self,active,stage,side,spacing):
        label=['stage #1','stage #2','stage #3','stage #4','stage #5','stage #6']
        self.active=active
        self.stage=stage
        self.name=label[stage-1]
        self.side=side
        self.spacing=spacing
        self.num=int(side/spacing)
        self.steps=long(spacing*250)
        origin_posix = common.lum.absolute_position(stage,2)
        origin_posiy = common.lum.absolute_position(stage,3)
        self.xy0=[origin_posix,origin_posiy]
        self.intensity=na.zeros((self.num,self.num),dtype='f4')
        self.xarray=[]
        self.yarray=[]
        if ( self.num % 2 == 0) or (self.num % 2 == 1): #n is even or odd
            for iii in range(int(self.num)):
                x_temp = long(origin_posix - self.steps * (self.num-1)/2.0 + self.steps * iii)
                y_temp = long(origin_posiy - self.steps * (self.num-1)/2.0 + self.steps * iii)
                if (x_temp > 0) and (x_temp < 128000):
                    self.xarray.append(x_temp)
                if (y_temp > 0) and (y_temp < 128000):
                    self.yarray.append(y_temp)
        else:
            print 'The range divided by the spacing must be an integer value'
            return (0,0,0,1)
        self.img_extent=na.zeros(4)
        self.img_extent=self.xarray[0]-0.5*self.steps,self.xarray[-1]+0.5*self.steps,self.yarray[0]-0.5*self.steps,self.yarray[-1]+0.5*self.steps 
        self.intensity_fit=na.zeros(1)
        self.fit_params=[]
        self.last_max=[0,0,0,0]
        self.last_gauss=[]
        self.last_gauss_display=[]

    def xy_move(self,index):
#        if na.greater_equal(index,self.num**2): return False
        ii=index//self.num
        jj=index%self.num
        if jj ==0:
            common.lum.move_absolute( self.stage, 3, self.yarray[ii])
        if ii%2==0:
            common.lum.move_absolute( self.stage, 2 ,self.xarray[jj])
        else:
            common.lum.move_absolute( self.stage, 2 ,self.xarray[-1-jj])
        return True
    
    def read_int(self,index,inten):
        if na.greater_equal(index,self.num**2): return False
        ii=index//self.num
        jj=index%self.num        
        if ii%2==0:
            self.intensity[ii][jj]=inten
        else:
            self.intensity[ii][-1-jj]=inten            
        return True
            


class mircMapping:
    def __init__(self):
	self.exit_index = 0 #declare signal index globally
        self.ircs = IRCamSocket.IRCamSocket()
        self.ircc = IRCamCommands.IRCamCommands()
#        self.mfex = mircFiberExplorerXchan.fiber()
        # flag for subtracting fluctuation of the whole detector. 
        #if it is 1, get_intensity function will return the flux of mirc quadrant minus idle quadrant
        #if it is 0, the function will return only mirc quadrant flux
        self.quad_subtract=1
#        self.num_stages=self.mfex.num_stages

    def stages_number(self,data):
        self.num_stages=data

    def z_prism(self, stage, z_length, z_spacing, side, spacing, ij_range = [0,256,0,256], seconds = .5):
	
	if((z_length * 10)%1 != 0) or ( (z_spacing*10)%1 != 0):
		print 'The range and spacing of z must be divisible by 100 nm (z resolution)'
	
	zn = (z_length/z_spacing) + 1 # zn is currently float
	
	z = [None]*int(zn)
	z0 = common.lum.absolute_position(stage,1)
	z_steps = long(z_spacing*10)
	
	if ( zn % 2 == 0) or (zn % 2 == 1): #n is even or odd
		for i in range(int(zn)):
			z[i] = long(z0 - z_steps * (zn-1)/2.0 + z_steps * i)
	else:
		print 'The range divided by the spacing must be an integer value'
		return [0],[0],[0]
	
	zn = int(zn)
	n = int(spacing/side)+1
	x = [None]*int(n)
	y = [None]*int(n)
	
	#intensities = [[intensity],[intensity],[intensity],...]
	intensities = [[[None]*2 for i in range(n)] for j in range(zn)]
	max_info = [[None]*3 for i in range(zn)]
	max_intensity_z = 0
	z_max = 0
	
	for i in range(zn):
            common.lum.move_absolute(stage,1,z[i])
            intensities[i],x,y = self.xy_square(stage, side, spacing, ij_range, seconds)
            max_info[i] = self.get_max(intensities[i],x,y)
            if max_info[i][2] > max_intensity_z:
                max_intensity_z = max_info[i][2]
                max_index = i
		
        common.lum.move_absolute(stage,1,z0)
	print 'Maximum intensity at z = ',z[max_index]
	print '                     x = ',max_info[max_index][0]
	print '                     y = ',max_info[max_index][1]
	print 'intensity = ',max_info[max_index][2]
	
	return [intensities, x, y, z] 
	
    def xy_total(self, stage, spacing, range_i = [0,256], range_j = [0,256], seconds = .5):
	if ((spacing * 1000) % 4 !=0):
            print 'The spacing must be divisible by 4 nm (xy resolution)'
            return 0
	
	common.lum.move_absolute(stage, 2, 62500)
	common.lum.move_absolute(stage, 3, 62500)
	return self.xy_square(stage, 500, spacing, ij_range, seconds)
	 

    def xy_square(self, active_stages, side, spacing, ij_range= [0,256,0,256],disp_img = 1,usingnewFE=1):
	
        n_stages=len(active_stages)

        rcParams['interactive'] = True
        #create a list to describe each stage
        all_stage=[]
        max_num=0
#        pdb.set_trace()
        for ii in range(n_stages):
            all_stage.append(Stage_Info(active_stages[ii],ii+1,side[ii],spacing[ii]))
            all_stage[ii].num = int(all_stage[ii].num)
            if all_stage[ii].num > max_num: max_num= all_stage[ii].num

            all_stage[ii].xarray=na.array(all_stage[ii].xarray)
            all_stage[ii].yarray=na.array(all_stage[ii].yarray)

	#initiate window for plotting
	if (disp_img):
            for ii in range(n_stages):
                if all_stage[ii].active ==0: continue
                #use figure 3 active figure
                pl.figure(3+ii)
                pl.clf()                       
                pl.title('Image of Intensity vs. Position, Current Map of '+all_stage[ii].name)
                pl.xlabel('X in steps')
                pl.ylabel('Y in steps')
                pl.ylabel('Y in steps') #advance the frame
	else:
            pass
		
	#set the signal handler
	signal.signal(signal.SIGTSTP,self.xy_interupt)
	print 'CTRL-Z to stop mapping'
	global exit_index
	exit_index = 0 #becomes 1 (true) when signal sent
	
	#nested for loops activate mappingxplt
	#first for loop gives y coordinate
	#second for loop gives x coordinate
	
        for index in range(max_num**2):
            if exit_index == 1: break
            
            #check if the stages are active and the index is out of range
            cont=False
            for ind_stage in range(n_stages):
                if all_stage[ind_stage].active==0:continue
                if na.greater_equal(index,all_stage[ind_stage].num**2):continue 
                all_stage[ind_stage].xy_move(index)
                cont=True
            if cont==False: break
            
            #get intensity for stages
            #pdb.set_trace()
            inten = self.get_intensity(ij_range,usingnewFE)
            for ind_stage in range(n_stages):
                if all_stage[ind_stage].active==0:continue
                if usingnewFE: 
                    all_stage[ind_stage].read_int(index,inten[ind_stage])
                else:
                    all_stage[ind_stage].read_int(index,inten)
                   
            
            #display image only one row is collected
            if disp_img==1:
                for ind_stage in range(n_stages):
                    if all_stage[ind_stage].active==0:continue
                    if na.greater_equal(index,all_stage[ind_stage].num**2): continue
                    ii=index//all_stage[ind_stage].num
                    jj=index%all_stage[ind_stage].num
                    if jj == (all_stage[ind_stage].num-1) :
                        pl.figure(3+ind_stage)
                        pl.imshow(all_stage[ind_stage].intensity,origin = 'lower',cmap =cm.hot,\
                                      vmin = all_stage[ind_stage].intensity[0:ii+1].min(), \
                                      vmax = all_stage[ind_stage].intensity[0:ii+1].max(), \
                                      extent = all_stage[ind_stage].img_extent,\
                                      interpolation='nearest',aspect='auto')
                        pl.ylabel('Y in steps')
        
                 

					
	#reset signal to default
	signal.signal(signal.SIGTSTP, signal.SIG_DFL)
			
	#move back to original location
	for ii in range(n_stages):
            if all_stage[ii].active==0:continue
            common.lum.move_absolute(all_stage[ii].stage,2,all_stage[ii].xy0[0])
            common.lum.move_absolute(all_stage[ii].stage,3,all_stage[ii].xy0[1])

	#close()
	
	#rcParams['interactive'] = False
		
	return all_stage,False



    def xy_interupt(self, signum,frame):
	print 'Interupt signal sent, mapping will end soon...'
	global exit_index
	exit_index = 1
	#sleep(1)
	return 

    def get_max(self, intensity,x,y):
	
	maximum = 0
	x_max_index = 0
	y_max_index = 0
	
	ny = len(y)
	nx = len(x)
	
	for i in range(ny):
		for j in range(nx):
			if intensity[i][j] > maximum:
				maximum = intensity[i][j]
				x_max_index = j
				y_max_index = i
	
	#move_absolute(stage,2,x[x_max_index])
	#move_absolute(stage,3,y[y_max_index])
	
	#del(dat)
		
	return [x[x_max_index],y[y_max_index],intensity[y_max_index][x_max_index]]
	
    def get_intensity_random(self):
	return randint(0,100)

    # get_intensity takes the range of pixels to be sampled and
    # returns the total intensity accross those pixels

    def get_intensity(self, ij_range = [0,256,0,256],usingnewFE =1):
        self.getNextFrame()
## only add up light in lower left quadrant! JDM
        if usingnewFE==0:
            #mirc quadrant
            quad_range0 = ij_range[0]
            quad_range1 = ij_range[1]/2 - 1
            quad_range2 = ij_range[2]
            quad_range3 = ij_range[3]/2 -1
            quad_range = [quad_range0, quad_range1, quad_range2, quad_range3]
            inten= common.dataArray[quad_range[0]:quad_range[1], quad_range[2]:quad_range[3]].sum()
            if self.quad_subtract==0: return inten

            #idle quadrant
            quad_range0 = ij_range[1]/2
            quad_range1 = ij_range[1]-1
            quad_range2 = ij_range[3]/2
            quad_range3 = ij_range[3]-1
            quad_range = [quad_range0, quad_range1, quad_range2, quad_range3]
            inten1 = common.dataArray[quad_range[0]:quad_range[1], quad_range[2]:quad_range[3]].sum()
            return inten-inten1

        else:
        
            inten=[]
            for ii in range(self.num_stages):
                temp=common.dataArray[0:7,self.xchan_pos[ii]-1:self.xchan_pos[ii]+1].sum()
                inten.append(temp)
            if self.quad_subtract==0: return inten

            inten1=[]
            inten2=[]
            for ii in range(self.num_stages):
                temp1=common.dataArray[0:7,self.xchan_pos[ii]-3:self.xchan_pos[ii]-2].sum()
                temp2=common.dataArray[0:7,self.xchan_pos[ii]+2].sum()
                inten[ii]=inten[ii]-temp1-temp2
            return inten

    def xchan_positions(self,data):
        self.xchan_pos=data

    def xy_surf(self, intensity,x,y):
	
	start_x = x[0]
	start_y = y[0]
	
	x_microns = [None] * len(x)
	y_microns = [None] * len(y)
	
	for i in range(len(x)):
		x_microns[i] = (x[i] - start_x) / 250.
	for i in range(len(y)):
		y_microns[i] = (y[i] - start_y) / 250.	
		
	gplt.surf(x_microns,y_microns,intensity)
	gplt.title('Intensity on camera vs. Luminos stage XY position')
	gplt.xtitle('X position in microns')
	gplt.ytitle('Y position in microns')
	gplt.ztitle('Intensity')
	gplt.hidden("remove")
	return

    def getNextFrame(self):
        #variable that is use to check if the available frame is a new frame
        common.coaddCount = self.ircc.coaddCount()
        coadd = common.coaddCount
        cnt = 0
        while coadd < common.coaddCount+2:
            coadd = self.ircc.coaddCount()
            cnt += 1
            time.sleep(.05)
        # a new frame is available, read the new frame
        self.ircs.readFrame()
        common.coaddCount = coadd
        if common.numberOfCumulatedImages>1:
            cnt = 0
            tmp = common.dataArray
            for i in range(common.numberOfCumulatedImages-1):
                while coadd == common.coaddCount:
                    coadd = self.ircc.coaddCount()
                    cnt += 1
                    time.sleep(.05)
                    # a new frame is available, read the new frame
                self.ircs.readFrame()
                common.coaddCount = coadd
                tmp += common.dataArray
            common.dataArray = tmp/common.numberOfCumulatedImages
        print "Received ",common.numberOfCumulatedImages," Frame(s)"

    def getFrame(self):
        #variable that is use to check if the available frame is a new frame
        coadd = common.coaddCount
        cnt = 0
        #img = na.zeros((common.numberOfCumulatedImages, common.rows, common.cols), type = na.Float32)
        tmp = na.zeros((common.rows, common.cols), type = na.Float32)
        for i in range(common.numberOfCumulatedImages):
            while coadd == common.coaddCount:
                coadd = self.ircc.coaddCount()
                cnt += 1
                time.sleep(.2)
                # a new frame is available, read the new frame
            self.ircs.readFrame()
            common.coaddCount = coadd
            tmp += common.dataArray
            #img[i, :, :] = common.dataArray
        common.dataArray = tmp/common.numberOfCumulatedImages
