import itertools
import struct
import numarray as na

'''
/* Big structure used for python configuration AND for 
sending info to the Spooler */
struct pythonconfig {
 int NTels;
 int BeamOrder[NUMTELS];
 int RefTel;
 unsigned int FiberExplorerX[NUMTELS];
 unsigned int FiberExplorerY[NUMTELS];
 float lambda;
 float bandwidth;
 float gdtThreshold[BASELINES];  //should really be in the gdt structure!
 float gdtGain;
 unsigned char cpeak[BASELINES];
 unsigned char cpeak0[BASELINES];
 unsigned char cpeak1[BASELINES];
 unsigned char bkgndNoiseCol;
 char fileDescription[30];
 char MircFilter1[30];
 char MircFilter2[30];
 char ConfigName[30];
 char TelKey[30];
 char observer[30];
 char comment1[30];
 char comment2[30];
};
typedef struct pythonconfig pythonconfig_struct;

pythonconfig_struct pyconfig = {
NUMTELS, //USED in COMBINER.
{0,0,0,0}, 
W2,
{0,0,0,0} , //FiberExplorerX
{0,0,0,0} , //FiberExplorerY
1.65,
0.3,
{1, 1, 2.5, 1, 1, 1},  //gdtThreshold
0.6   ,//gdtGain
{0,0,0,0,0,0},
{0,0,0,0,0,0},
{0,0,0,0,0,0},
0,
"DATA                          ", // File Description
"6. H Band                     ", // MIRC Filter 1
"1. OPEN                       ", // MIRC Filter 2
"H_PRISM                       ", // Config Name
"S1=0,S2=1,E1=2,E2=3,W1=4,W1=5 ", // TelKey
"Slimfringe /0                ", // Observer
"Seeing is crap.               ", // comment1
"And W2 is oscillating crazy  !", // comment2
};
'''
NUMTELS = 4
BASELINES = 6
S1,S2,E1,E2,W1,W2 = 0,1,2,3,4,5

#the python config structure
pythonConfigKeys = ['NTels','BeamOrder','RefTel','FiberExplorerX','FiberExplorerY', \
                    'lambda','bandwidth','gdtThreshold','gdtGain', \
                    'cpeak','cpeak0','cpeak1','bkgndNoiseCol',\
                    'fileDescription','MircFilter1','MircFilter2','ConfigName',\
                    'TelKey','observer','comment1', 'comment2']

pythonConfigValues = [NUMTELS,na.array([0,0,0,0]),W2,na.array([0,0,0,0]),na.array([0,0,0,0]),\
                        1.65,0.3,na.array([1., 1., 2.5, 1., 1., 1.]),0.6,\
                        na.array([0,0,0,0,0,0]),na.array([0,0,0,0,0,0]),na.array([0,0,0,0,0,0]),0,\
                        "DATA                          ",\
                        "6. H Band                     ",\
                        "1. OPEN                       ",\
                        "H_PRISM                       ",\
                        "S1=0,S2=1,E1=2,E2=3,W1=4,W2=5 ",\
                        "Slimfringe                    ", \
                        "Seeing is crap.               ",\
                        "And W2 is oscillating crazy  !"]
pythonConfig = dict(itertools.izip(pythonConfigKeys,pythonConfigValues))
pythonConfigFormat ='i%dii%dI%dIff%dff%dB%dB%dBB30s30s30s30s30s30s30s30s'%(NUMTELS,NUMTELS,NUMTELS,BASELINES,BASELINES,BASELINES,BASELINES)
pythonConfigSize = struct.calcsize(pythonConfigFormat)

def stringToIntArray(string):
    return na.array([int(i.strip('[,]')) for i in string.split()])

def stringToFloatArray(string):
    return na.array([float(i.strip('[,]')) for i in string.split()])

def floatToString(f):
    return "%2.2f"%f

def floatArrayToString(farr):
    string = '['
    for f in farr:
        string += "%2.2f "%f
    l = len(string)
    return string[:l-1]+']'

def doNothing(string):
    return string.strip('\x00')

pythonConfigToString = [str, str, str, str, str, \
                        floatToString, floatToString, floatArrayToString, floatToString,\
                        str, str, str, str, \
                        doNothing, doNothing, doNothing, doNothing, doNothing, doNothing, doNothing, doNothing]

pythonConfigFromString = [int, stringToIntArray, int, stringToIntArray, stringToIntArray,\
                        float, float, stringToFloatArray, float, \
                        stringToIntArray, stringToIntArray, stringToIntArray, int,\
                        doNothing, doNothing, doNothing, doNothing, doNothing, doNothing, doNothing, doNothing]

def packPythonConfig(ustructValues):
    pstruct  = struct.pack('i', ustructValues[0])
    pstruct += struct.pack('%di'%(NUMTELS), *ustructValues[1])
    pstruct += struct.pack('i', ustructValues[2])
    pstruct += struct.pack('%dI'%(NUMTELS), *ustructValues[3])
    pstruct += struct.pack('%dI'%(NUMTELS), *ustructValues[4])
    pstruct += struct.pack('f', ustructValues[5])
    pstruct += struct.pack('f', ustructValues[6])
    pstruct += struct.pack('%df'%(BASELINES), *ustructValues[7])
    pstruct += struct.pack('f', ustructValues[8])
    pstruct += struct.pack('%dB'%(BASELINES), *ustructValues[9])
    pstruct += struct.pack('%dB'%(BASELINES), *ustructValues[10])
    pstruct += struct.pack('%dB'%(BASELINES), *ustructValues[11])
    pstruct += struct.pack('B', ustructValues[12])
    pstruct += struct.pack('30s', ustructValues[13])
    pstruct += struct.pack('30s', ustructValues[14])
    pstruct += struct.pack('30s', ustructValues[15])
    pstruct += struct.pack('30s', ustructValues[16])
    pstruct += struct.pack('30s', ustructValues[17])
    pstruct += struct.pack('30s', ustructValues[18])
    pstruct += struct.pack('30s', ustructValues[19])
    pstruct += struct.pack('30s', ustructValues[20])
    return pstruct 

def unpackPythonConfig(data):
    posi = 0
    pythonConfig['NTels'], = struct.unpack('i', data[posi:posi+4])
    posi += 4
    pythonConfig['BeamOrder'] = na.array((struct.unpack('%di'%(NUMTELS), data[posi:posi+NUMTELS*4])))
    posi += NUMTELS*4 
    pythonConfig['RefTel'], = struct.unpack('i', data[posi:posi+4])
    posi += 4
    pythonConfig['FiberExplorerX'] = na.array((struct.unpack('%dI'%(NUMTELS), data[posi:posi+NUMTELS*4])))
    posi += NUMTELS*4
    pythonConfig['FiberExplorerY'] = na.array((struct.unpack('%dI'%(NUMTELS), data[posi:posi+NUMTELS*4])))
    posi += NUMTELS*4
    pythonConfig['lambda'], = struct.unpack('f', data[posi:posi+4])
    posi += 4
    pythonConfig['bandwidth'], = struct.unpack('f', data[posi:posi+4])
    posi += 4
    pythonConfig['gdtThreshold'] = na.array(struct.unpack('%df'%(BASELINES), data[posi:posi+BASELINES*4]))
    posi += BASELINES*4
    pythonConfig['gdtGain'], = struct.unpack('f', data[posi:posi+4])
    posi += 4
    pythonConfig['cpeak'] = na.array(struct.unpack('%dB'%(BASELINES), data[posi:posi+BASELINES]))
    posi += BASELINES
    pythonConfig['cpeak0'] = na.array(struct.unpack('%dB'%(BASELINES), data[posi:posi+BASELINES]))
    posi += BASELINES
    pythonConfig['cpeak1'] = na.array(struct.unpack('%dB'%(BASELINES), data[posi:posi+BASELINES]))
    posi += BASELINES
    pythonConfig['bkgndNoiseCol'], = struct.unpack('B', data[posi])
    posi += 1
    pythonConfig['fileDescription'], = struct.unpack('30s', data[posi:posi+30])
    posi += 30
    pythonConfig['MircFilter1'], = struct.unpack('30s', data[posi:posi+30])
    posi += 30
    pythonConfig['MircFilter2'], = struct.unpack('30s', data[posi:posi+30])
    posi += 30
    pythonConfig['ConfigName'], = struct.unpack('30s', data[posi:posi+30])
    posi += 30
    pythonConfig['TelKey'], = struct.unpack('30s', data[posi:posi+30])
    posi += 30
    pythonConfig['observer'], = struct.unpack('30s', data[posi:posi+30])
    posi += 30
    pythonConfig['comment1'], = struct.unpack('30s', data[posi:posi+30])
    posi += 30
    pythonConfig['comment2'], = struct.unpack('30s', data[posi:posi+30])
    #for ndx, key in enumerate(pythonConfigKeys):
        #print key, pythonConfig[key]
    return pythonConfig
    
'''
struct theDaq {
unsigned short rows;
unsigned short cols;
unsigned short rowOffset;
unsigned short colOffset;  
unsigned short nreads;
unsigned short numberOfCoadds;
unsigned short framesPerReset;
unsigned short exptime;
unsigned long numberOfFrames;
unsigned char byteSwapping;
unsigned char do_reset;
unsigned char do_deinterlace;
unsigned char do_image_save;
unsigned char daqRunning;
unsigned char readToggle;
float voffset;
float vreset;  
float vd;
float ictl;
char image_directory[30];
char image_filename[20];
unsigned char continuousReadout;
unsigned short savenum;
unsigned short numberCoherent;
};

'''

theDaqKeys = ['rows', 'cols', 'rowOffset', 'colOffset', 'nreads', 'numberOfCoadds', 'framesPerReset', \
                        'exptime', 'numberOfFrames', 'byteSwapping', 'do_reset', 'do_deinterlace', 'do_image_save', \
                        'daqRunning', 'readToggle', 'voffset', 'vreset', 'vd', 'ictl', 'image_directory', \
                        'image_filename', 'continuousReadout', 'savenum', 'numberCoherent']
theDaqValues = [16, 160, 5, 38, 8, 15, 1000, \
                0, 1000, 0, 1, 0, 0, \
                0, 0, 4.0, 0.6, 5.0, 3.3, '/data/', \
                'mirc.fit', 1, 0, 10]
theDaq  = dict(itertools.izip(theDaqKeys, theDaqValues))
theDaqFormat = '8HL6B4f30s20sB2H'
theDaqSize = struct.calcsize(theDaqFormat)

theDaqType = [int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,\
                        float,float,float,float,str,str,int,int,int]

daqToString = [str, str, str, str, str, str, str, str, str, str, str, str, str, str, str, \
                        floatToString, floatToString, floatToString, floatToString, \
                        doNothing, doNothing, str, str, str]

daqFromString = [int,int,int,int,int,int,int,int,int,int,int,int,int,int,int,\
                        float,float,float,float,str,str,int,int,int]

def packTheDaq(ustructValues):
    return struct.pack(theDaqFormat, *ustructValues)

def unpackTheDaq(data):
    values = list(struct.unpack('8HL6B4f30s20sB2H',data))
    theDaq  = dict(itertools.izip(theDaqKeys, values))
    theDaq['image_directory'] = theDaq['image_directory'].strip('\x00')
    theDaq['image_filename'] = theDaq['image_filename'].strip('\x00')
    #for ndx, key in enumerate(theDaqKeys):
        #print key, theDaq[key]
    return theDaq

